#include    "p18f67j50.h"
#include 	"lcd/lcd.h"
#include    "compass/compass.c"
//#include    "mmc.h"
#include    <math.h>
        
        #pragma config XINST    = OFF		// Extended instruction set
        #pragma config STVREN   = OFF		// Stack overflow reset
        #pragma config PLLDIV   = 5			// (20 MHz crystal used on this board)
//		#pragma config PLLDIV   = 2			// (8 MHz INT crystal used on this board)
        #pragma config WDTEN    = OFF		// Watch Dog Timer (WDT)
        #pragma config CP0      = OFF		// Code protect
        #pragma config CPUDIV   = OSC1		// OSC1 = divide by 1 mode
        #pragma config IESO     = OFF		// Internal External (clock) Switchover
        #pragma config FCMEN    = OFF		// Fail Safe Clock Monitor
        #pragma config FOSC     = HSPLL		// Firmware must also set OSCTUNE<PLLEN> to start PLL!
//		#pragma config FOSC     = INTOSCPLLO
        #pragma config WDTPS    = 32768
//      #pragma config WAIT     = OFF		// Commented choices are
//      #pragma config BW       = 16		// only available on the
//      #pragma config MODE     = MM		// 80 pin devices in the 
//      #pragma config EASHFT   = OFF		// family.
        #pragma config MSSPMSK  = MSK5
//      #pragma config PMPMX    = DEFAULT
//      #pragma config ECCPMX   = DEFAULT
        #pragma config CCP2MX   = DEFAULT

#define GPGGA 394
#define GPRMC 413

#define RES_HIGH		PORTDbits.RD3=1
#define RES_LOW			PORTDbits.RD3=0

#define MENU_NUMB   3
#define KEY_NONE    0
#define KEY_UP      1
#define KEY_DOWN    2
#define KEY_LEFT    3
#define KEY_RIGHT   4
#define KEY_CENTER  5

///////////////////////////////////////VARIABLES////////////////////////////////////////////
char buffer[81];
char buffer_pos=0;
long int lat, lon, alt; // latitude longtitude altitude
char   N, E;            // direction symbols
float course;           // course in degrees relative to North 
float speed;            // speed over ground in m/s
char read_flag=0;       // buffer overflow
char coord[15];         // print string
int temp;               // should be removed
long int log_pos=0;

unsigned long hlat=42091830, hlon=24463541;    // home latitude and longtitude
unsigned char JoyPos;
unsigned char MenuPos=1;
unsigned char  offset=0, menu_flag=0;
unsigned char Inversion [] ={0,0,0,0,0,0};
//unsigned char TestMMC_e[]={"MMC error\0"};
//unsigned char TestMMC_s[]={"MMC success\0"};
unsigned char CP[]={"MMC Not Present\0"};
unsigned char WP[]={"MMC Write Protect\0"};

#pragma udata IBM_APP_RAM3
//extern char mmc_buffer[128];
//extern char mmc_buffer_test_2[128];

#pragma idata IBM_APP_RAM2
unsigned char Menu[MENU_NUMB][15] = {
 "Status Screen \0",
 "Compass \0",
 "Homing Screen \0",
// "Logger Screen \0",
};
//rom 
extern char mmc_buffer_test_1[128];
 char state_mmc  = 1;

#pragma udata IBM_APP_RAM
long int distance;
long int delta_lat;
long int delta_lon;
float temp_cos;
unsigned long tlat, tlon;
char log_flag=0;

////////////////////////////////////////////////////////////////////////////////////////////
void get_buffer(void);
void read_buffer(void);
//void log_buffer(char);
#pragma code lowVector=0x18
void InterruptUARTLow (void) 
{  _asm    goto get_buffer  _endasm
}
#pragma code
////////////////////////////////////////////////////////////////////////////////////////////

#pragma interrupt get_buffer 
void get_buffer()  // get the message from the GPS module
{
  if(PIR1bits.RC1IF) // check if uart filled
  {
    buffer[buffer_pos]=RCREG1; // get info
    buffer_pos++;
    if(buffer[buffer_pos-1]==10) 
    {
    //   if(log_flag) log_buffer(buffer_pos-1); 
       read_buffer();
       buffer_pos=0; 
    }
  }  
   PIR1 &= ~0x20; // clear interruption
   PIE1 |= 0x20;
} 

void InitLED2(void)
		{ 
			//RE3
			TRISE = TRISE & 0b11110111;
			PORTE = PORTE & 0b11110111;
		}
//asynchronos reception
void InitUART()
{ 
    TRISC |= 0x80;    		// set RC7 as a reciever
    TRISC &= ~0x40;   		// set RC6 as a transmiter
    SSPCON1 = 0;      		// disable PSI or sth
    SPBRG1 = 0x9B;    		// 155 dec
	SPBRGH1 = 0x00;   		// 0x0271 for 48MHz -> 19200 baud
    TXSTA1 = 0x24;    		// begin transmission, BRGH=1;
    OSCCON |= 0b01110000; 
    RCSTA1 |= 0x90;   		// asynchronous continuous mode
    BAUDCON &= ~0x08; 		// BRG16 = 0
    PIE1   |= 0x20;   		// enable interrupts
    RCSTA1 |= 0x10;   		// begin reception 
    INTCON |= 0xC0;   		// enable global and peripherial interrupts    
    OSCTUNE |= 0x40;  		// enable PLL
}
   
void read_buffer()  // dissect the message in the buffer
{ 
   //   1       2         3       4          5         6          7 8    9   10   11 12  13 14 15
   // $GPGGA,101624.000,4209.2194,N        ,02446.6210,E         ,1,03  ,2.3,124.2,M,37.1,M,   ,0000*5E
   // $GPRMC,101623.000,A        ,4209.2194,N         ,02446.6210,E,0.00,194.42,230708,,,A*6D

     
    int ID=0;         // ID of the NMEA message  
    int i;            // buffer position   
    int j;            // secondary buffer position  
    int segment=1;    // part which is read (between two commas)
    char dec_flag;   
    for(i=0; i<6; i++) ID += (int)buffer[i];
    
    switch(ID)
    {
       case GPRMC:
       {
         segment=1;
          while(segment<13)  
          {
             switch(segment)
             {
                case 8: // speed over ground segment
                {
                   j=i;
                   speed=0;
                   while(buffer[j]!=',')
                   {
                      if(buffer[j]=='.') {dec_flag=1; j++; continue; }
                      if(dec_flag) dec_flag++;
                      speed*=10;
                      speed+=buffer[j]-'0';
                      j++;
                   }        
                   while(dec_flag>1) {speed/=10; dec_flag--;}
                   dec_flag=0;           
                   speed*=0.51444;  // convert in m/s
                   i=j;
                   break;
                }
                case 9: // course over ground segment
                {
                   j=i;
                   course=0;
                   while(buffer[j]!=',')
                   {
                      if(buffer[j]=='.') {dec_flag=1; j++; continue;}
                      if(dec_flag) dec_flag++;
                      course*=10;
                      course+=buffer[j]-'0';
                      j++;
                   }        
                   while(dec_flag>1) {course/=10; dec_flag--;}  
                   dec_flag=0;  
                   i=j;       
                   break;
                }
             }

             if(buffer[i]==',') segment++; 
             i++;
          }
          break;
       }
       case GPGGA:
       {
          segment=1; 
          while(segment<14)  
          {  
             switch(segment)
             {
                case 3:   // latitude segment
                {
                   j=i;
                   lat=0;
                   while(buffer[j]!=',')
                   {
                      if(buffer[j]=='.') {j++; continue; }
                      lat*=10;
                      lat+=buffer[j]-'0';
                      j++;
                   }  
                   i=j;  
                   break;
                }
                case 4:  // lattitude symbol segment
                {
                   N=buffer[i]; 
                   i++;
                   break;
                }
                case 5: // longtitude segment
                {
                   j=i;
                   lon=0;
                   while(buffer[j]!=',')
                   {
                      if(buffer[j]=='.') {j++; continue;}
                      lon*=10;
                      lon+=buffer[j]-'0';
                      j++;
                   }   
                   i=j;                
                   break;
                }
                case 6:  // longtitude symbol segment
                { 
                   E=buffer[i];
                   i++;             
                   break;
                }
                case 10: // altitude segment
                {
                   j=i;
                   alt=0;
                   while(buffer[j]!=',')
                   {
                      if(buffer[j]=='.') { j++; continue;}
                      alt*=10;
                      alt+=buffer[j]-'0';
                      j++;
                   }     
                   alt/=10;
                   i=j;  
                   break;
                } 
             }

             if(buffer[i]==',') segment++; 
             i++;
          }
          break;
       }
    } 
}

void clear_coord()
{
   int i;
   for(i=0; i<16; i++) coord[i]=32;
}
// reads the coordinates to a format ready for output
// the information is stored in coord[]
void read_coord(char cmd)
{
    
    int i;
    long int k;
    clear_coord();
    if(cmd=='N' || cmd == 'S') k=lat - (lat%10000)*0.4;
    if(cmd=='E' || cmd == 'W') k=lon - (lon%10000)*0.4;
    if(cmd=='P') k=hlat;  // home coordinates
    if(cmd=='L') k=hlon;
    if(cmd=='A')
    {
      coord[0]='A';
      coord[1]='l';
      coord[2]='t';
      coord[3]='i';
      coord[4]='t';
      coord[5]='u';
      coord[6]='d';
      coord[7]='e';
      coord[8]=':';
      coord[9]=' ';

      k=alt;
      
      for(i=12; i>=10; i--)
      {
         coord[i]=k%10+'0';
         k/=10;
      }
      coord[13]='m';
    }
    if(cmd=='V')
    {
      coord[0]='S';
      coord[1]='p';
      coord[2]='e';
      coord[3]='e';
      coord[4]='d';
      coord[5]=':';
      coord[6]=' ';
      k=speed*10;
      
      for(i=10; i>=7; i--)
      {
         if(i==9){coord[i]='.'; continue;}
         coord[i]=k%10+'0';
         k/=10;
      }
      coord[11]='m';
      coord[12]='/';
      coord[13]='s';  
    }
    if(cmd=='C')
    {
      coord[0]='C';
      coord[1]='o';
      coord[2]='u';
      coord[3]='r';
      coord[4]='s';
      coord[5]='e';
      coord[6]=':';
      coord[7]=' ';
      k=course*10;
      
      for(i=12; i>=8; i--)
      {
         if(i==11){coord[i]='.'; continue;}
         coord[i]=k%10+'0';
         k/=10;
      }
      coord[13]='~';  
    }
    if(cmd=='D')
    {
      coord[0]='D';
      coord[1]='i';
      coord[2]='s';
      coord[3]='t';
      coord[4]=':';
      coord[5]=' ';
      k=distance;
      
      for(i=12; i>=6; i--)
      {
         if(i==11){coord[i]='.'; continue;}
         coord[i]=k%10+'0';
         k/=10;
      }
      coord[13]='m';  
    }
    if(cmd=='N' || cmd == 'S' || cmd=='E' || cmd == 'W' || cmd=='P' || cmd == 'L')
    {
    	for(i=13; i>=1; i--)
    	{
         	if(i==13){ coord[i]='"'; continue;}
         	if(i==10){ coord[i]='.'; continue;}
         	if(i==7){ coord[i]='\''; continue;}
         	if(i==4){ coord[i]='~'; continue;}
         	coord[i]=k%10+'0';
         	k/=10;
    	}
    	if(coord[1]=='0') coord[1]=' ';
    	coord[0]=cmd;
        if(cmd=='P') coord[0]='N';
        if(cmd=='L') coord[0]='E';
	}
    coord[14]='\0';
}

/*
void log_buffer(char num)
{
   int i;
   // copy the buffer
   for(i=0; i<=num; i++)
   {
       mmc_buffer[i]=buffer[i];
   }

   mmcWriteBlock(log_pos);   
   mmcReadBlock(log_pos, 512);
   log_pos+=512;  
    
}*/
////////////////////////////////////////////////////////////////////////////////////////////
        int joystick_right(void)
		{
		//RA4
			if (!(PORTA & 0b00010000))
				return 1;
			else return 0;
		}
		
		int joystick_left(void)
		{
		//RB1
			if (!(PORTB & 0b00000010))
				return 1;
			else return 0;
		}
		
		int joystick_select(void)
		{
		//RB0
	       if (!(PORTB & 0b00000001))
				return 1;
			else return 0;
		}
 		
		int joystick_up(void)
		{
		//RB4
		if (!(PORTB & 0b00010000))
				return 1;
			else return 0;
		}
 		
		int joystick_down(void)
		{
		//RB5
		if (!(PORTB & 0b00100000))
				return 1;
			else return 0;
		}

unsigned char GetJoystickPosition (void) { 
  if (joystick_up() && joystick_left()&& joystick_select()) 
      { return KEY_UP; }
  if (joystick_down() && joystick_right()&& joystick_select())
     { return KEY_DOWN; }
  if (joystick_left()&& joystick_select() && joystick_down()) 
	  { return KEY_LEFT; }
  if (joystick_right()&& joystick_select()&& joystick_up())  
	{ return KEY_RIGHT; }
  if (joystick_select()) return KEY_CENTER;
  return KEY_NONE;

}

void UpdateMenu(unsigned char pos) {

  Inversion[pos-1] = 1;
  LCDUpdate();
  LCDClear();
  LCDStr ( 0, Menu[0+offset], Inversion[0]);
  LCDStr ( 1, Menu[1+offset], Inversion[1]);
  LCDStr ( 2, Menu[2+offset], Inversion[2]);
  LCDStr ( 3, Menu[3+offset], Inversion[3]);
//  LCDStr ( 4, Menu[4+offset], Inversion[4]);
//  LCDStr ( 5, Menu[5+offset], Inversion[5]);
//  Delay(100000);	

  Inversion[pos-1] = 0;
}

long int step;
void main(void)
{
    TRISEbits.TRISE3 = 0; //Led outputs
	TRISEbits.TRISE2 = 0;
  
   // LCDInit();
   //	LCDContrast(0x45);
   //	LCDUpdate();
	
    InitLED2();
    InitUART();

    LCDInit();
	LCDContrast(0x45);
	LCDClear();
	LCDUpdate();

    while(1)
    {
start: UpdateMenu(MenuPos);
       while(1)
	   {
			JoyPos = GetJoystickPosition();
			
			if(JoyPos==KEY_UP)
			 {
			   	MenuPos--;
			    if((MenuPos<1)) MenuPos = 1;
                UpdateMenu(MenuPos);
                Delay(150000);
			 }	
				
			 if(JoyPos==KEY_DOWN)
              {
			      MenuPos++;
			      if((MenuPos>3)) MenuPos = 3;
			      UpdateMenu(MenuPos);
                  Delay(150000);
			  }
			  
              if(JoyPos==KEY_RIGHT)
              {
                 Delay(150000);
				 break;	
			  }
   	   } 

skip:  if(RCSTA1bits.OERR==1)
       {
             RCSTA1 &= ~0x10;
             RCSTA1 |= 0x10; 
       } 
         
       // Status screen
       if(MenuPos == 1)
	   {
            LCDClear();
            Delay(500);
            LCDUpdate();
        	
            read_coord(N);
      		LCDStr (0, coord, 0);
      		Delay(1000);
      		read_coord(E);
      		LCDStr (1, coord, 0);
      		Delay(1000);
      		read_coord('A');          //altitude
      		LCDStr (2, coord, 0);
      		Delay(1000);
      		read_coord('V');          //velocity
      		LCDStr (3, coord, 0); 
      		Delay(1000);
      		read_coord('C');          //course
      		LCDStr (4, coord, 0); 
            
      }
      //Compass screen 
      if(MenuPos == 2)
      {    
            draw_compass((int)course, 1);      
      		read_coord('C');        //course
      		LCDStr (5, coord, 0);
	        Delay(150000); 
      }

      // Homing screen
      if(MenuPos == 3)
      {    
            step=0xF4240;  // 1 000 000
            //input
            do
            {
               LCDClear();
               LCDUpdate();
             //  coord="   Set base \0";
             //  LCDSt (0, coord, 0);
              // coord="  coordinates \0";
              // LCDSt (1, coord, 0);  
               read_coord('P');
               LCDStr (2, coord, 0);
               read_coord('L');
      		   LCDStr (3, coord, 0);
               
               Delay(60000);
               while(1)
	           {
					JoyPos = GetJoystickPosition();
                    Delay(1000);
			        if(JoyPos==KEY_UP)
			 		{
                        hlat += step;
			    		break;
			 		}	
				
					if(JoyPos==KEY_DOWN)
                    {		  		    
						hlat -= step;
			    		break;
			  		}
			
                    if(JoyPos==KEY_RIGHT)
				    {
                        step/=100; 
                        Delay(150000);
                        break;
                    } 
                    if(JoyPos==KEY_CENTER)
			 		{
			   	        // get the current coordinates
                        hlat = lat - (lat%10000)*0.4;
                        step=0;
			    		break;
			 		} 
                    if(JoyPos==KEY_LEFT) goto start;           				
   	           }  
                         
      		} while(step>0);
            
            step=0xF4240;
            do
            {
               LCDClear();
               LCDUpdate();
            
             //  LCDSt (0, "   Set base \0", 0);
             //  coord="  coordinates \0";
             //  LCDSt (1, coord, 0);  
               read_coord('P');
               LCDStr (2, coord, 0);
               read_coord('L');
      		   LCDStr (3, coord, 0);

               Delay(60000);
               while(1)
	           {
					JoyPos = GetJoystickPosition();
			        Delay(1000);
					if(JoyPos==KEY_UP)
			 		{
			   			hlon+=step;
			    		break;
			 		}	
				
					if(JoyPos==KEY_DOWN)
              		{
			  		   	hlon-=step;
			    		break;
			  		}
			
                    if(JoyPos==KEY_RIGHT)
				    {
                      step/=100; 
                      Delay(150000);
                      break;
                    }       
                    if(JoyPos==KEY_CENTER)
			 		{
			   	        // get the current coordinates
                        hlon = lon - (lon%10000)*0.4;
                        step=0;
			    		break;
			 		} 
                    if(JoyPos==KEY_LEFT) goto start;     				
   	           }  
                 
      		}while(step>0);   
            do  // display arrow and distance
            {
                 
                 tlat=lat - (lat%10000)*0.4;
                 delta_lat=((hlat/1000000)*3600000+((hlat/10000)%100)*6000+((hlat/100)%100)*100+(hlat%100))-
                           ((tlat/1000000)*3600000+((tlat/10000)%100)*6000+((tlat/100)%100)*100+(tlat%100)) ;
                 tlon = lon - (lon%10000)*0.4;
 				 delta_lon= ((hlon/1000000)*3600000+((hlon/10000)%100)*6000+((hlon/100)%100)*100+(hlon%100))- 
                            ((tlon/1000000)*3600000+((tlon/10000)%100)*6000+((tlon/100)%100)*100+(tlon%100));

                 delta_lat*=3; // one second corespondes to 30 meters
                 delta_lon*=3;  
                 temp=(int)tlat/1000000;
                 delta_lon*=cosa((int)tlat/1000000); // *cos(avg(delta latitude))
                 distance=sqrt((delta_lat*delta_lat + delta_lon*delta_lon)*1.0);
                 
                 temp_cos=(float)delta_lon/distance;
                
                 if(delta_lon>=0)  // I or IV qudrant
                 {
                    if(temp_cos<0)temp_cos+=2*temp_cos;
                    for(step=0; step<46; step++) 
                      if(coss[step]<=temp_cos)  
                      {  
                         if(delta_lat>=0) draw_compass(90-step*2, 2);  //I
                           else draw_compass(90+step*2, 2);         //IV
                         break;
                      }
                 }
                 else

                 {
                    if(temp_cos<0)temp_cos+=2*temp_cos;
                    for(step=0; step<46; step++) 
                      if(coss[step]<=temp_cos)  
                      {  
                        if(delta_lat>=0) draw_compass(270+step*2, 2);  //II
                          else draw_compass(270-step*2, 2);         //III
                        break;
                      }
                 } 
        
                 read_coord(N);
                 LCDStr (3, coord, 0);
                 read_coord(E);
      		     LCDStr (4, coord, 0); 
                 read_coord('D');
                 LCDStr (5, coord, 0);   
                 Delay(150000);
                 JoyPos = GetJoystickPosition();
                    if(JoyPos==KEY_LEFT) goto start;  

            }while(1);
	  }
      // Logger Screen write to MMC Card
      if(MenuPos == 4)
      {
            TRISEbits.TRISE0=1;
			TRISEbits.TRISE1=1;
/*	     	if((PORTEbits.RE1==0))
			{		
					if((PORTEbits.RE0==0))
					{      
				    		if((initMMC() == MMC_SUCCESS));	// card found
							{	
								memset(&mmc_buffer,0x00,128);
								
								mmcReadRegister (10, 16);
		    					mmc_buffer[7]=0;
					
		    					// Fill first Block (0) with 'A'
		   						memset(&mmc_buffer,'0',128);    //set breakpoint and trace mmc_buffer contents
								mmcWriteBlock(0);
		    					// Fill second Block (1)-AbsAddr 512 with 'B'
		    					memset(&mmc_buffer,'1',128);
								mmcWriteBlock(512);
					
		   			 			// Read first Block back to buffer
		   						memset(&mmc_buffer,0x00,128);
		    					mmcReadBlock(0,512);
								memset(&mmc_buffer_test_1,'0',128);
		    					if(strncmp(&mmc_buffer[0], &mmc_buffer_test_1[0], 128))
				 					state_mmc=1;
								else state_mmc  = 0;
		   
		    					memset(&mmc_buffer,0x00,128); //set breakpoint and trace mmc_buffer contents
		    					mmcReadBlock(512,512);
								memset(&mmc_buffer_test_1,'1',128);
		    					if(strncmp(&mmc_buffer[0], &mmc_buffer_test_1[0], 128)) 
									state_mmc=1;
								else state_mmc  = 0;
				
		    					memset(&mmc_buffer,0x00,128); //set breakpoint and trace mmc_buffer contents 	
			 				}	
			 		}
				else  
                { state_mmc=2; LCDStr(0,WP,0); Delay(900000);	goto start;}
			}
			else 
			{state_mmc=3;	LCDStr(0,CP,0);	Delay(900000); goto start;} 
           
           log_flag=1;
           do
           {
                 if(RCSTA1bits.OERR==1)
      			 {
             		RCSTA1 &= ~0x10;
            	    RCSTA1 |= 0x10; 
       			 } 
                 LCDClear();
                 LCDUpdate();
                 coord[0]='L'; coord[1]='o';coord[2]='g';coord[3]='g';coord[4]='i';coord[5]='n';coord[6]='g';
                 coord[7]=' '; coord[8]=' ';coord[9]=' ';coord[10]=' ';coord[11]=' ';coord[12]=' ';coord[13]=' ';
                 coord[14]='\0'; 
                 LCDStr (0, coord, 0);
                 read_coord(N);
                 LCDStr (1, coord, 0);
                 read_coord(E);
      		     LCDStr (2, coord, 0); 
 
                 Delay(150000);
                 JoyPos = GetJoystickPosition();
                    if(JoyPos==KEY_LEFT){log_flag=0; goto start;}  
           }while(1);
      */
     
      } 
   	  Delay(150000);
      JoyPos = GetJoystickPosition();
            if(JoyPos==KEY_LEFT) goto start;
            else goto skip;
    } 
    return;
}
